/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.pv168.jdbcdemopeople;

import java.io.IOException;
import java.sql.Connection;
import static java.sql.DriverManager.getConnection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import javax.sql.DataSource;
import org.apache.commons.dbcp2.BasicDataSource;

/**
 * To run this demo, you need a standalone Java DB (Derby) server instance
 * listening on port 1527 at localhost.
 *
 * Database People must be created before (you can do it from the SQL console ij
 * from command line or from NetBeans SQL console. Then create table Person with
 * columns name (varchar) and salary (int).
 *
 * @author tomp
 */
public class JDBCDemoPeople {

    private static String driver = "org.apache.derby.jdbc.ClientDriver";
    private static String dbURL = "jdbc:derby://localhost:1527/";
    private static String dbName = "People";

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws SQLException, ClassNotFoundException, InstantiationException, IllegalAccessException, Exception {
        loadDriver();

        demoDelete();
        // demoInsert();
        // demoSelectIncorrect();
        // demoSelect();
        // demoSelectPreparedStatement();
        demoSelectTemplate();

    }

    private static void loadDriver() throws ClassNotFoundException, InstantiationException, IllegalAccessException {
        Class.forName(driver).newInstance();
    }

    // incorrect: it does not handle exceptions, it throws them
    private static void demoSelectIncorrect() throws SQLException {
        Connection connection = getConnection(dbURL + dbName + ";create=true");
        Statement st = connection.createStatement();
        boolean result = st.execute("SELECT * FROM Person");
        if (result) {
            ResultSet rs = st.getResultSet();
            while (rs.next()) {
                String name = rs.getString("name");
                int salary = rs.getInt("salary");
                System.out.println("Person " + name + " has salary " + salary);
            }
        }
        // incorrect: it does not close the connection!
    }

    // recommended way since Java 7 (try with resources)
    private static void demoSelect() {
        // correct: it automatically closes the connection
        try (Connection connection = getConnection(dbURL + dbName + ";create=true")) {
            Statement st = connection.createStatement();
            boolean result = st.execute("SELECT * FROM Person");
            if (result) {
                ResultSet rs = st.getResultSet();
                while (rs.next()) {
                    String name = rs.getString("name");
                    int salary = rs.getInt("salary");
                    System.out.println("Person " + name + " has salary " + salary);
                }
            }
        } catch (SQLException e) {
            System.err.println("cannot select people " + e.getLocalizedMessage());
        }
    }

    // even better, prepared statementString is more secure
    private static void demoSelectPreparedStatement() {
        String myName = "Tomáš Pitner";
        // correct: it automatically closes the connection
        try (Connection connection = getConnection(dbURL + dbName + ";create=true")) {
            // correct: it automatically closes the statementString if bad
            try (PreparedStatement st = connection.prepareStatement("SELECT * FROM Person WHERE name = ?")) {
                // setting the parameters this way prevents SQL Injection
                st.setString(1, myName);
                // from this point it stays the same
                boolean result = st.execute();
                if (result) {
                    ResultSet rs = st.getResultSet();
                    while (rs.next()) {
                        int salary = rs.getInt("salary");
                        System.out.println("Person " + myName + " has salary " + salary);
                    }
                }
            }
        } catch (SQLException e) {
            System.err.println("cannot select people " + e.getLocalizedMessage());
        }
    }

    private static void demoInsert() {
        String newName = "Jan Novák";
        int newSalary = 20000;
        try (Connection connection = getConnection(dbURL + dbName + ";create=true")) {
            try (PreparedStatement insertStatement = connection.prepareStatement(
                    "INSERT INTO Person (name,salary) VALUES (?,?)")) {
                // set the actual parameters
                insertStatement.setString(1, newName);
                insertStatement.setInt(2, newSalary);
                // performs the actual insert 
                int updateCount = insertStatement.executeUpdate();
                System.out.println("Rows inserted " + updateCount);
            }
        } catch (SQLException e) {
            System.err.println("cannot insert to people " + e.getLocalizedMessage());
        }
    }

    private static void demoDelete() {
        String nameToDelete = "Jan Novák";
        try (Connection connection = getConnection(dbURL + dbName + ";create=true")) {
            try (PreparedStatement deleteStatement = connection.prepareStatement(
                    "DELETE FROM Person WHERE name = ?")) {
                // set the actual parameters
                deleteStatement.setString(1, nameToDelete);
                // performs the actual delete 
                int updateCount = deleteStatement.executeUpdate();
                System.out.println("Rows deleted " + updateCount);
            }
        } catch (SQLException e) {
            System.err.println("cannot delete from people " + e.getLocalizedMessage());
        }
    }

    private abstract static class SQLSelectTemplate {

        public void execute() throws Exception {
            try (Connection connection = getConnection(dbURL + dbName + ";create=true")) {
                try (PreparedStatement statement = connection.prepareStatement(statementString())) {
                    // performs the actual select
                    final ResultSet rs = statement.executeQuery();
                    processResultSet(rs);
                }
            } catch (SQLException e) {
                handleException(e);
            }
        }

        public abstract String statementString();

        public void processResultSet(ResultSet executeQuery) throws SQLException {
        }

        public void handleException(SQLException e) throws Exception {
            throw e;
        }
    }

    // even better, prepared statementString is more secure
    private static void demoSelectTemplate() throws Exception {
        SQLSelectTemplate template = new SQLSelectTemplate() {
            @Override
            public String statementString() {
                return "SELECT * FROM Person WHERE name = 'Tomáš Pitner'";
            }
            @Override
            public void processResultSet(ResultSet rs) throws SQLException {
                while(rs.next()) {
                    System.out.println(rs.getString(1));
                }
            }
        };
        template.execute();
    }
    
     public static DataSource dataSource() throws IOException {
        Properties p =  new Properties();
        p.load(JDBCDemoPeople.class.getResourceAsStream("/jdbc.properties"));
 
        BasicDataSource bds = new BasicDataSource(); //Apache DBCP connection pooling DataSource
        bds.setDriverClassName(p.getProperty(driver));
        bds.setUrl(p.getProperty(dbURL+dbName));
        return bds;
    }
}
